-- Below are some global variables that control various aspects of the game, feel free to change
zombie_team = 1 -- 0 is red, 1 is blue
zombie_speed = 1.5 -- zombie speed
lastman_speed = 1.5 -- last man speed
lastman_dmgmodifier = 1.5 -- damage modifier for the last man (on top of human damage)
human_speed = 1.0 -- speed when not infected
zombie_count = 0.20 -- if value is less than 1 is it used as a percentage, more than or equal to one is absolute count
last_man_next_zombie = true -- if this value is true the last man standing becomes the next zombie, if not it's random
zombie_message = "YOU'RE A ZOMBIE. KILL THE HUMANS!"
human_message = "YOU'RE A HUMAN. SURVIVE."
welcome_message = "Welcome. This server runs on Phasor. http://haloapps.wordpress.com"
rejoin_message = "Please don't leave and rejoin. You've been put back onto your last team."
infected_message = " has been infected!"
teamkill_message = "Don't team kill..."
blockteamchange_message = "You're not allowed to change team."
zombieinvis_message = "The zombies are invisible for 20 seconds!"
zombievehicle_message = "Zombies aren't allowed to use vehicles."

-- Don't modify below variables unless you know what you're doing
cur_blue_count = 0
cur_red_count = 0
cur_players = 0
cur_last_man = -1
last_man_hash = 0
last_team = 255
processid = 0
game_started = false
allow_change = false
hash_table = {}
new_game_timer = 0
player_change_timer = 0

-- This function is required for scripts to run on Phasor (including and after 01.00.03.104)
-- You should return the minimum required version of Phasor
-- Official releases:
--   01.00.03.104 - 3104
--   01.00.10.057 - 10057
function GetRequiredVersion()
	return 10057
end

-- called when the script is loaded
function OnScriptLoad(process)

	processid = process

	-- assume a game is running
	game_started = true


	cur_red_count = getteamsize(0)
	cur_blue_count = getteamsize(1)

	cur_players = cur_red_count + cur_blue_count

	-- load the last man hash
	local file = io.open("lasthash_" .. processid .. ".tmp", "r")

	if (file ~= nil) then

		-- read the hash
		last_man_hash = file:read("*line")

		file:close()

		-- delete the file
		os.remove("lasthash_" .. processid .. ".tmp")
	end
end

-- called when the script is unloaded
-- Do not return a value
function OnScriptUnload()
end

-- called when a game is starting (before any players join)
-- Do not return a value.
function OnNewGame(map)

	-- reset our variables
	cur_blue_count = 0
	cur_red_count = 0
	cur_players = 0

	-- the game hasn't started yet, will once timer runs down
	game_started = false
	new_game_timer = registertimer(2000, "NewGameTimer")
end

-- called when a game is ending
-- Do not return a value.
function OnGameEnd(mode)
	-- mode 1 = score menu (F1) is being displayed to clients, they are still ingame
	-- mode 2 = post game menu appeared
	-- mode 3 = players can quit via the post game score card

	-- remove the new game timer (if still active)
	if mode == 1 then
		removetimer(new_game_timer)
	end
end

-- Called when there is chat within the server
-- It must return a value which indicates whether or not the chat is sent.
function OnServerChat(player, chattype, message)
	return 1 -- allow all chat, 0 would block the chat
end

-- Called when a server command is being executed
-- It must return a value which indicates whether or not the command should be processed
-- Note: It is only called when an authenticated (and allowed) player attempts to use the command.
--		 player is -1 when being executed via the server console.
function OnServerCommand(player, command)
	return 1 -- allow it
end

-- Called when a player's team is being chosen as the join the server.
-- It must return a value which indicates the team the player is to be put on.
-- Note: this function being called doesn't guarantee that the player will be able to successfully join.
function OnTeamDecision(cur_team)

	local decision = cur_team

	-- check if the number of zombies has been met
	local finalZombies = 0

	if zombie_count >= 1 then
		finalZombies = zombie_count
	else
		finalZombies = round(cur_players * zombie_count)
	end

	curCount = 0

	-- check which team are the zombies
	if zombie_team == 0 then
		curCount = cur_red_count
	else
		curCount = cur_blue_count
	end

	-- check if the numer of zombies has been met
	--if curCount == 0 or (curCount < finalZombies and last_team ~= zombie_team) then
	if curCount == 0 or curCount < finalZombies then

		decision = zombie_team

	else

		if zombie_team == 0 then
			decision = 1
		else
			decision = 0
		end

		last_team = decision
	end

	return decision
end

-- Called when a player joins the server.
-- Do not return a value.
function OnPlayerJoin(player, team)

	-- update the player counts
	cur_players = cur_players + 1

	local thisTeamSize = 0
	if team == 0 then
		cur_red_count = cur_red_count + 1
		thisTeamSize = cur_red_count
	else
		cur_blue_count = cur_blue_count + 1
		thisTeamSize = cur_blue_count
	end

	local thisHash = gethash(player)
	local alreadyExists = false

	-- check if the player has joined this game
	for k,v in pairs(hash_table) do
		if thisHash == k then

			if thisTeamSize > 1 then
				if v ~= team then changeteam(player, 0)	end

				--hprintf("A player has left and later rejoined (hash " .. thisHash .. ")")
				privatesay(player, rejoin_message)

				team = v
			end
			alreadyExists = true

			break
		end
	end

	--hprintf("thisTeamSize " .. thisTeamSize)
	if alreadyExists == false then
		hash_table[thisHash] = team
	end

	-- send a welcome message to the new player
	--name = getname(player)

	privatesay(player, welcome_message)

	if game_started == true then

		-- check if the player is a zombie
		if team == zombie_team then

			makezombie(player)

			privatesay(player, zombie_message)

		else

			-- check if we were at last man
			if cur_last_man ~= -1 then
				makehuman(cur_last_man)
				cur_last_man = -1
			end

			makehuman(player)
			privatesay(player, human_message)

		end
	end
end

-- Called when a player the server.
-- Do not return a value.
function OnPlayerLeave(player, team)

	if team == 0 then
		cur_red_count = cur_red_count - 1
	elseif team == 1 then
		cur_blue_count = cur_blue_count - 1
	end

	cur_players = cur_players - 1

	if game_started == true then

		local zombiecount = getzombieteamsize()
		local humancount = gethumanteamsize()

		if cur_players > 1 then

			if zombiecount == 0 then
				noZombiesLeft()
			elseif humancount == 0 then
				all_players_zombies(-1)
			end
		end
	end

	-- update team within table
	local thisHash = gethash(player)
	hash_table[thisHash] = team
end

-- called when a player kills another, 'killer' is the index of the killing player, 'victim' is the index of the dead player
-- Do not return a value.
function OnPlayerKill(killer, victim, mode)

	if game_started then

		if mode == 0 then -- server kill
		elseif mode == 1 then -- fall damage
		elseif mode == 2 then -- killed by guardians
		elseif mode == 3 then -- killed by vehcile
		elseif mode == 4 then -- killed by another player

			local killer_team = getteam(killer)

			-- check to see if a zombie killed them
			if killer_team == zombie_team then

				-- get the player's name
				name = getname(victim)

				-- change their team
				changeteam(victim, 0)

				-- set zombie attributes
				makezombie(victim)

				-- notify the server
				say(name .. infected_message)

				privatesay(victim, zombie_message)
			end

		elseif mode == 5 then -- betrayed
			privatesay(killer, teamkill_message)
		elseif mode == 6 then -- suicide
		end
	end
end

-- called when a player gets a double kill, killing spree etc
-- see Phasor documentation for multiplier values
function OnKillMultiplier(player, multiplier)
end

-- Called when a player spawns (after object is created, before players are notified)
-- Do not return a value
function OnPlayerSpawn(player, m_objectId)

	local team = getteam(player)

	if team == zombie_team then

		-- lookup the object
		local m_object = getobject(m_objectId)

		if m_object ~= nil then

			local zombSize = getzombieteamsize()

			--if zombSize > 7 then
			-- make zombies spawn with 1 grenade
			--	writebyte(m_Object, 0x31E, 1)
			--	writebyte(m_Object, 0x31F, 0)
			--end

			-- spawn with no ammo
			--for i=0,3 do
			--	local m_weaponId = readdword(m_object, 0x2F8 + (i*4))
			--	if m_weaponId ~= 0xffffffff then

					-- get the weapons memory address
			--		local m_weapon = getobject(m_weaponId)

			--		if m_weapon ~= nil then
						-- set the ammo
			--			writeword(m_weapon, 0x2B6, 0)
			--			writeword(m_weapon, 0x2B8, 0)
						-- force it to sync
			--			updateammo(m_weaponId)
			--		end
			--	end
			--end
		end
	end
end

-- Called after clients have been notified of a player spawn
-- Do not return a value
function OnPlayerSpawnEnd(player, m_objectId)

end

-- Called when a player is attempting to change team.
-- A value must be returned. The return value indicates whether or not the player is allowed the change team.
-- Notes: If relevant is 1 the return value is considered, if it's 0 then the return value is ignored.
function OnTeamChange(relevant, player, team, dest_team)

	if relevant == 1 and allow_change == false then

		privatesay(player, blockteamchange_message)

		-- we don't let people change team
		return 0

	else
		-- we can't stop the person changing teams, bein done by an admin
		if dest_team == 0 then
			cur_red_count = cur_red_count + 1
			cur_blue_count = cur_blue_count - 1
		elseif dest_team == 1 then
			cur_red_count = cur_red_count - 1
			cur_blue_count = cur_blue_count + 1
		end

		-- check if the game has started yet
		if game_started == true then

		-- change the players speed
			if dest_team == zombie_team then
				makezombie(player)

			else
				makehuman(player)
			end

			-- check if the game should end
			if zombie_team == 0 then
				if cur_blue_count == 0 then
					all_players_zombies(player)
				elseif cur_blue_count == 1 then
					onlastman(1)
				end
			elseif zombie_team == 1 then
				if cur_red_count == 0 then
					all_players_zombies(player)
				elseif cur_red_count == 1 then
					onlastman(0)
				end
			end
		end

		if allow_change == true then

			allow_change = false

			-- remove change timer
			removetimer(player_change_timer)
		end

		-- update team within table
		local thisHash = gethash(player)
		hash_table[thisHash] = dest_team
	end

	return 1
end

-- Called when a player interacts with an object
-- It can be called while attempting to pick the object up
-- It is also called when standing above an object so can be called various times quickly
function OnObjectInteraction(player, m_ObjectId, tagType, tagName)

	local response = 1

	if game_started == true then

		-- regardless of the team, stop people taking the flag
		if tagType == "weap" then
			if tagName == "weapons\\flag\\flag" then
				response = 0
			end
		end

		if response == 1 then

			local team = getteam(player)

			if team == zombie_team then

				-- we want to stop zombies picking up weapons
				if tagType == "weap" then
					-- only let zombies get shotgun ammo
					if tagName ~= "weapons\\shotgun\\shotgun" then
						response = 0
					end

				elseif tagType == "eqip" then

					if tagName == "weapons\\frag grenade\\frag grenade" or tagName == "weapons\\plasma grenade\\plasma grenade" then
						response = 0 -- don't let them pick the nades
					elseif tagName == "powerups\\active camouflage" then

						-- check if the picking player is already invisible
						local m_player = getplayer(player)

						if m_player ~= nil then

							local m_playerObjId = readdword(m_player, 0x34)

							local m_object = getobject(m_playerObjId)

							if m_object ~= nil then

								local camoFlag = readdword(m_object, 0x204)

								if camoFlag ~= 0x51 then

									-- check if zombies are to be made invisible
									local doInvis = getrandomnumber(1, 10)

									--hprintf("doInvis " .. doInvis)

									if doInvis > 5 then

										-- make the whole zombie team invis for 20 seconds
										for x = 0,15 do

											local team = getteam(x)

											if team == zombie_team and x ~= player then
												--hprintf("applying camo to player " .. x)
												applycamo(x, 20.00)
											end
										end

										say(zombieinvis_message)
									end
								end
							end
						end
					end

				end
			end

		end
	end
	return response
end

-- Called when a player attempts to reload their weapon.
-- A value must be returned. The return value indicates whether or not the player is allowed to reload.
-- Notes: If player is -1 then the weapon being reload wasn't located (it could be a vehicle's weapon)
function OnWeaponReload(player, weapon)

	local response = 1

	--hprintf("Player " .. player .. " is reloading weapon " .. weapon)

	return response
end


-- Called when a player attempts to enter a vehicle.
-- A value must be returned. The return value indicates whether or not the player is allowed to enter.
function OnVehicleEntry(player, vehicleId, vehicle_tag, seat)

	local response = 1
	if game_started then

		-- check if they're a zombie
		local team = getteam(player)

		if team == zombie_team then
			response = 0 -- cannot enter vehicle

			privatesay(player, zombievehicle_message)
		end

	end

	return response
end

-- Called when damage is being done to an object.
-- This doesn't always need to be a player.
-- Do not return a value
function OnDamageLookup(receiving_obj, causing_obj, tagdata, tagname)

	-- Resolve the object ids into player ids
	-- Remember, the damage doesn't always need to be done by a player.
	local receiver = objecttoplayer(receiving_obj)
	local causer = objecttoplayer(causing_obj)

	-- We want to amplify human damage, so make sure the causer is human
	if causer ~= nil then

		-- Get the player's team
		local c_team = getteam(causer)

		-- Make sure the team was found
		if c_team ~= nil then

			-- Check if it's a human causing the damage
			if c_team ~= zombie_team then

				-- It's human damage, read the current damage the weapon has
				local min_dmg = readfloat(tagdata, 0x1D0)
				local max_dmg_min = readfloat(tagdata, 0x1D4)
				local max_dmg_max = readfloat(tagdata, 0x1D8)

				-- This is used to increase human damage (for last man)
				local modifier = 1.0

				-- Check if we're up to last man
				if cur_last_man ~= -1 then
					-- Change the damage modifier to the last man's
					modifier = lastman_dmgmodifier
				end

				-- Change the damage that will be done
				writefloat(tagdata, 0x1D0, min_dmg * 2 * modifier)
				writefloat(tagdata, 0x1D4, max_dmg_min * 2 * modifier)
				writefloat(tagdata, 0x1D8, max_dmg_max * 2 * modifier)

			-- It's a zombie causing the damage
			elseif c_team == zombie_team then

				-- check if it is melee damage, if so increase it to instant kill
				local found = string.find(tagname, "melee", -5)

				-- We only want to increase zombie's melee damage
				if found ~= nil then

					-- make melee instant kill
					writefloat(tagdata, 0x1D0, 500)
					writefloat(tagdata, 0x1D4, 500)
					writefloat(tagdata, 0x1D8, 500)

				end
			end
		end
	end
end

-- Called when a player is being assigned a weapon (usually when they spawn)
-- A value must be returned. The return value is the id of the weapon they're to spawn with. Return zero if the weapon shouldn't change.
-- Notes: 	This is called for all weapon spawns the player has
--			This is also called with player 255 when vehicles are being assigned weapons.
-- 			This is only considered if in gametype, the weapon set is 'generic' if not this has no effect.
function OnWeaponAssignment(player, object, count, tag)

	local actualWeapon = 0

	-- check if the player is a zombie
	local team = getteam(player)

	if team == zombie_team then
		-- we only want zombies to have shotguns
		actualWeapon = lookuptag("weap", "weapons\\shotgun\\shotgun")

	end

	return actualWeapon
end

-- Called when an object is created
-- Do not return a value.
function OnObjectCreation(m_objectId, player_owner, tag)

end

-- The below functions are used internally within the Lua script
function NewGameTimer(id, count)

	if count == 5 then

		say("The game is starting...")

		if cur_players ~= 0 then

			cur_red_count = getteamsize(0)
			cur_blue_count = getteamsize(1)

			local cur_zombie_count = 0

			-- by default make all players human
			for x=0,15 do
				local thisTeam = getteam(x)

				-- check if the team was found
				if thisTeam ~= -1 then
					if thisTeam == zombie_team then
						changeteam(x, 0)
					end

				end
			end

			local last_man_index = -1

			-- check if the last man is to be made a zombie
			if last_man_next_zombie == true then

			--hprintf("last man next zombie")
				-- loop through all hashes and check if any match the last man
				for i=0,15 do
				  local hash = gethash(i)

				  if hash ~= nil then
					-- check to see if they were last man
					if last_man_hash == hash then

						--hprintf("Last man found at " .. i .. " hash " .. hash)
						makezombie(i)

						-- change their team
						changeteam(i, 0)

						cur_zombie_count = 1
						last_man_index = i

						break
					end
				  end
				end
			end

			last_man_hash = 0;

			-- select other zombies (if necessary) randomly
			local finalZombies = 0

			if zombie_count >= 1 then
				finalZombies = zombie_count
			else
				finalZombies = round((cur_players * zombie_count) + 0.5)
			end

			if finalZombies == 0 then
				finalZombies = 1
			elseif finalZombies == curPlayers then
				finalZombies = finalZombies - 1
			end


			-- set counters such that changeteam wont end
			cur_red_count = 16
			cur_blue_count = 16

			--hprintf("Expecting " .. finalZombies .. " zombies. Cur players " .. cur_players)
			-- randomly choose players to make zombies
			while (cur_zombie_count < finalZombies) do

				-- randomly choose a player
				local newzomb = ChooseRandomPlayer(zombie_team)

				if newzomb ~= last_man_index then

					makezombie(newzomb)

					--hprintf("changing team to zombie " .. newzomb)

					-- change their team
					changeteam(newzomb, 0)

					cur_zombie_count = cur_zombie_count + 1
				end

			end

			-- set team counters
			if zombie_team == 0 then
				cur_red_count = finalZombies
				cur_blue_count = cur_players - finalZombies
			else
				cur_red_count = cur_players - finalZombies
				cur_blue_count = finalZombies
			end

			-- reset the map
			svcmd("sv_map_reset")

			-- loop through and tell players which team they're on
			for i=0,15 do

				-- get the player's team
				local pteam = getteam(i)

				if pteam ~= nil then
					-- check if they're a zombie
					if pteam == zombie_team then
						privatesay(i, zombie_message)
					else
						privatesay(i, human_message)
					end
				end
			end

		end

		game_started = true

		return 0 -- remove timer
	else
		say("The game will start in " .. 10 - 2*count .. " seconds.")

		return 1 -- keep timer
	end

end

function PlayerChangeTimer(id, count)

	if count ~= 5 then

		local zombsize = getzombieteamsize()

		if allow_change == false or zombsize > 0 then

			allow_change = false

			say("Thank you, the game can continue.")

			return 0
		end

		say("In " .. 5 - count .. " seconds a player will be forced to become a zombie.")

		return 1
	else -- timer up, force team change

		allow_change = false

		local newZomb = ChooseRandomPlayer(zombie_team)

		--hprintf("making player " .. newZomb .. " a zombie")

		local oldHumanCount = 0

		if zombie_team == 0 then
			oldHumanCount = cur_blue_count
			cur_blue_count = 16
		else
			oldHumanCount = cur_red_count
			cur_red_count = 16
		end

		changeteam(newZomb, 1)
		makezombie(newZomb)

		if zombie_team == 0 then
			cur_blue_count = oldHumanCount - 1
		else
			cur_red_count = oldHumanCount - 1
		end

		return 0
	end
end

-- this function searches through current players and selects a random one
function ChooseRandomPlayer(excludeTeam)

	local t = {}

	-- loop through all 16 possible spots and add to table
	for i=0,15 do

		-- check if the player exists
		local team = getteam(i)

		if team ~= nil and team ~= excludeTeam then
			table.insert(t, i)
		end

	end

	-- generate a random number that we will use to select a player
	local tableCount = 	table.getn(t)
	local r = getrandomnumber(1, tableCount+1)
	return t[r]
end

function makezombie(player)

	-- change the player's speed
	setspeed(player, zombie_speed)

end

function makehuman(player)

	setspeed(player, human_speed)
end

-- there are no zombies left
function noZombiesLeft()

	allow_change = true

	say("There are no zombies left. Someone please change team, otherwise someone will be forced to change.")

	player_change_timer = registertimer(1000, "PlayerChangeTimer")

end

-- called when its the last man
function onlastman(humanTeam)

	-- lookup the last man
	for x=0,15 do
		local team = getteam(x)

		if team == humanTeam then

			-- give the last man speed and extra ammo
			setspeed(x, lastman_speed)

			cur_last_man = x

			-- find the last man's weapons
			local m_player = getplayer(x)

			if m_player ~= nil then
				local m_ObjId = readdword(m_player, 0x34)

				-- find the player's object
				local m_object = getobject(m_ObjId)

				if m_object ~= nil then
					-- give all weapons 600 ammo
					for i=0,3 do
						local m_weaponId = readdword(m_object, 0x2F8 + (i*4))

						if m_weaponId ~= 0xffffffff then

							-- get the weapons memory address
							local m_weapon = getobject(m_weaponId)

							if m_weapon ~= nil then
								-- set the ammo
								writeword(m_weapon, 0x2B6, 600)
							end
						end
					end
				end
			end
		elseif team ~= nil then
			-- kill the player
			--kill(x)
		end
	end

	if cur_last_man ~= -1 then

		local lastman_name = getname(cur_last_man)

		say(lastman_name .. " is the last man alive and is invisible for 15 seconds!")

		applycamo(cur_last_man, 15.0)
	end


end

-- called when all players are zombies (no shit)
function all_players_zombies(player)

	local local_red = getteamsize(0)
	local local_blue = getteamsize(1)

	if player ~= -1 then
		--hprintf("Local sizes (red then blue) " .. local_red .. " " .. local_blue)
		--hprintf("Tracked sizes (red then blue) " .. cur_red_count .. " " .. cur_blue_count)

		--hprintf("Last man player: " .. player .. " name " .. getname(player))
		-- get hash of last man
		last_man_hash = gethash(player)
		--hprintf("Last man hash: " .. last_man_hash)

		-- write the hash to file
		local file = io.open("lasthash_" .. processid .. ".tmp", "w")

		if (file ~= nil) then
			file:write(last_man_hash)
			file:close()
		end
	end

	-- move onto the next map
	svcmd("sv_map_next")
end

function getzombieteamsize()
	local ret_val = 0

	if zombie_team == 0 then
		ret_val = cur_red_count
	else
		ret_val = cur_blue_count
	end

	return ret_val
end

function gethumanteamsize()
	local ret_val = 0

	if zombie_team == 1 then
		ret_val = cur_red_count
	else
		ret_val = cur_blue_count
	end

	return ret_val
end

function round(num)
    under = math.floor(num)
    upper = math.floor(num) + 1
    underV = -(under - num)
    upperV = upper - num
    if (upperV > underV) then
        return under
    else
        return upper
    end
end
