-- how long they are invulnerable for (ms)
invulnerable_duration = 3000

-- This function is required for scripts to run on Phasor (including and after 01.00.03.104)
-- You should return the minimum required version of Phasor
-- Official releases:
--   01.00.03.104 - 3104
--	 01.00.10.057 - 10057
function GetRequiredVersion()
	return 10057
end

-- called when the script is loaded
-- process is the id of the current process
function OnScriptLoad(process)
end

function OnScriptUnload()
end

function OnNewGame(map)
end

function OnGameEnd(mode)
end

function OnTeamDecision(team)
	return team
end

function OnPlayerJoin(player, team)
end

function OnPlayerLeave(player, team)
end

-- Called when there is chat within the server
-- It must return a value which indicates whether or not the chat is sent.
function OnServerChat(player, chattype, message)
	return 1 -- allow all chat, 0 would block the chat
end

-- Called when a server command is being executed
-- It must return a value which indicates whether or not the command should be processed
-- Note: It is only called when an authenticated (and allowed) player attempts to use the command.
--		 player is -1 when being executed via the server console.
function OnServerCommand(player, command)
	return 1
end

function OnTeamChange(relevant, player, cur_team, dest_team)
	return 1
end

-- Called when an update packet is received from a client
-- This function is called about 30 times every second for each client
-- Be careful NOT to use ANY IO (file) operations in here. It will lag.
--Arguments: arg1 = id of player, arg2 = object id of player
-- Return value: none
function OnClientUpdate(player, m_objectId)
end

function OnPlayerSpawn(player, m_objectId)

	local m_object = getobject(m_objectId)
	if m_object ~= nil then
		-- ignore collisions with this obj
		writebit(m_object, 0x10, 7, 1)

		registertimer(invulnerable_duration, "RemoveSpawnProtection", m_object)
	end
end

-- Called after clients have been notified of a player spawn
-- Do not return a value
function OnPlayerSpawnEnd(player, m_objectId)

end

-- Called when a player is being assigned a weapon (usually when they spawn)
-- A value must be returned. The return value is the id of the weapon they're to spawn with.
-- Return 0 if the weapon shouldn't change.
-- CAUTION: RETURNING AN INCORRECT VALUE WILL CAUSE THE SERVER TO CRASH
function OnWeaponAssignment(player, object, order, tag)
	return 0
end

-- Called when a player interacts with an object
-- It can be called while attempting to pick the object up
-- It is also called when standing above an object so can be called various times quickly
function OnObjectInteraction(player, m_ObjectId, tagType, tagName)

	local response = 1
	return response
end

-- Called when an object is created
-- Do not return a value.
function OnObjectCreation(m_objectId, player_owner, tag)
end

-- Called when damage is being done to an object.
-- This doesn't always need to be a player.
-- Do not return a value
function OnDamageLookup(receiving_obj, causing_obj, tagdata, tagname)
end

function OnVehicleEntry(relevant, player, vehicleId, vehicle_tag, seat)
	return 1
end

function OnVehicleEject(player, forceEject)
	return 1
end

-- called when a player kills another, 'killer' is the index of the killing player, 'victim' is the index of the dead player
-- 'killer' should be checked for validity (between 0 and 15) before use.
-- Do not return a value.
function OnPlayerKill(killer, victim, mode)
end

-- called when a player gets a double kill, killing spree etc
function OnKillMultiplier(player, multiplier)
end

-- Called when a player attempts to reload their weapon.
-- A value must be returned. The return value indicates whether or not the player is allowed to reload.
-- Notes: If player is -1 then the weapon being reload wasn't located (it could be a vehicle's weapon)
function OnWeaponReload(player, weapon)
	return 1
end

function RemoveSpawnProtection(id, count, m_object)
	writebit(m_object, 0x10, 7, 0)
	return 0
end
